"""
Defines storage interfaces and default implementations.
"""

from .map_storage import MapStorage
from .wallet_storage import WalletStorage
from .mongo_transaction_attempt_list_storage_impl import MongoTransactionAttemptListStorageImpl
from .mongo_map_storage_impl import MongoMapStorageImpl
from .mongo_wallet_storage_impl import MongoWalletStorageImpl
from .mongo_key_value_storage_impl import MongoKeyValueStorageImpl
from .coin_block_height_storage_proxy_impl import CoinBlockHeightStorageProxyImpl
from .waves_block_height_storage_proxy_impl import WavesBlockHeightStorageProxyImpl
from .key_value_storage import KeyValueStorage
from .block_height_storage_proxy import BlockHeightStorageProxy
from .transaction_attempt_list_storage import TransactionAttemptListStorage
from .coin_polling_state_storage_proxy_impl import CoinPollingStateStorageProxyImpl
from .waves_polling_state_storage_proxy_impl import WavesPollingStateStorageProxyImpl
from .polling_state_storage_proxy import PollingStateStorageProxy
